@@ -81,6 +81,12 @@ impl Signature {
         Ok(())
     }
     
+    pub fn param_count(&self) -> usize {
+        self.required.len()
+        + self.default.len()
+        + usize::from(self.variadic().is_some())
+    }
+    
     /// Get the length of the argument buffer required by bind_args()
     pub fn arg_len(&self) -> usize {
         self.required.len() + self.default.len()
